local ldb = LibStub:GetLibrary("LibDataBroker-1.1")
local ADDON_NAME, namespace = ...
local L = namespace.L
local build_version, build_number, build_date, build_tocversion = GetBuildInfo()
local isLegion = build_tocversion == 70000

-- Forward declares
local GetLootSpecInfo, CreateDataObject, UpdateSpecInfo, OnEvent
local specObject, eventFrame
local activeSpecName, activeSpecIndex, activeSpecIcon

local function GetLootSpecInfo()
	local id, name, description, icon, background, role, class
	local isAuto = false
	local lootSpecId = GetLootSpecialization()
	if lootSpecId == 0 then
		local currentSpec = GetSpecialization()
		if currentSpec ~= nil then
			local specId = GetSpecializationInfo(currentSpec)
			id, name, description, icon, background, role, class = GetSpecializationInfoByID(specId)
			isAuto = true
		else
			return nil
		end
	else
		id, name, description, icon, background, role, class = GetSpecializationInfoByID(lootSpecId)
	end

	return id, name, description, icon, background, role, class, isAuto
end

local function GetSpecInfo()
	local currentSpec = GetSpecialization()
	if currentSpec ~= nil then
		local specId = GetSpecializationInfo(currentSpec)
		return GetSpecializationInfoByID(specId)
	else
		return nil
	end
end

-- local function OnTooltipShow(tooltip)
-- 	if InCombatLockdown() then return end

-- 	local id, name, description, icon, background, role, class = GetSpecInfo()
	
-- 	if description ~= nil then
-- 		tooltip:AddLine(description, 1, 1, 1, true)
-- 	end

-- 	if role ~= nil then
-- 		tooltip:AddLine("Role: " .. role)
-- 	end

-- 	if class ~= nil then
-- 		tooltip:AddLine("Class: " .. class)
-- 	end

-- 	if background ~= nil then
-- 		tooltip:AddTexture(background)
-- 	end
-- end

local function UpdateActiveSpec()
	local id, name, description, icon, background, role, class = GetSpecInfo()
	
	activeSpecIndex = GetSpecialization()
	activeSpecName = name
	activeSpecIcon = icon
end

local function OnClick(self, button)

	if button == 'LeftButton' then
		if InCombatLockdown() then return end
		if UnitLevel("player") < SHOW_SPEC_LEVEL then return end

		local numSpecs
		if isLegion then
			numSpecs = GetNumSpecializations()
		else
			numSpecs = GetNumSpecGroups()
		end

		if numSpecs > 1 then
			activeSpecIndex = activeSpecIndex + 1
			if activeSpecIndex > numSpecs then
				activeSpecIndex = 1
			end

			if isLegion then
				SetSpecialization(activeSpecIndex)
			else
				SetActiveSpecGroup(activeSpecIndex)
			end
		end
	elseif button == 'RightButton' then
		local newSpecIndex, newSpecId

		local currentSpecIndex = GetSpecialization()
		local specCount = GetNumSpecializations()
		local currentLootSpecId = GetLootSpecialization()

		if(currentLootSpecId == 0) then
			newSpecIndex = 1
		else
			for specIndex = 1, specCount do
				local id = GetSpecializationInfo(specIndex)
				if(id == currentLootSpecId) then
					newSpecIndex = specIndex + 1
				end
			end
		end

		if newSpecIndex > specCount then
			newSpecIndex = 0
		end

		if newSpecIndex == 0 then
			newSpecId = 0
		else
			newSpecId = GetSpecializationInfo(newSpecIndex)
		end

		SetLootSpecialization(newSpecId)
	end
end

local function CreateDataObject()
	return ldb:NewDataObject("Caerdon Spec Broker", {
		type = "data source",
		text = "",
		-- TODO: If I'm going to use the tooltip, fix the tooltip refresh when clicking to change spec
		-- OnTooltipShow = OnTooltipShow,
		OnClick = OnClick
	})
end

local function UpdateSpecInfo()
	UpdateActiveSpec()

	local dataText = activeSpecName

	local id, name, description, icon, background, role, class, isAuto = GetLootSpecInfo()
	if name ~= nil then
		if activeSpecName ~= name then
			dataText = dataText .. " (" .. L["Loot: "] .. name .. ")"
		elseif isAuto then
			dataText = dataText .. "*"
		end
		specObject.text = dataText
		specObject.icon = activeSpecIcon
	else
		specObject.text = L["Loot: "] .. L["None"]
		specObject.icon = ""
	end
end

local function OnEvent(frame, event, ...)
	if event == "ADDON_LOADED" then
		if ... == ADDON_NAME then
			if IsLoggedIn() then
				OnEvent(eventFrame, "PLAYER_LOGIN")
			else
				eventFrame:RegisterEvent "PLAYER_LOGIN"
			end
		end
	elseif event == "PLAYER_LOGIN" then
		eventFrame:RegisterEvent "PLAYER_LOOT_SPEC_UPDATED"
		eventFrame:RegisterEvent "ACTIVE_TALENT_GROUP_CHANGED"
		UpdateSpecInfo()
	elseif event == "PLAYER_LOOT_SPEC_UPDATED" then
		UpdateSpecInfo()
	elseif event == "ACTIVE_TALENT_GROUP_CHANGED" then
		UpdateSpecInfo()
	else
		print("EVENT RECEIVED: " .. event)
	end
end

eventFrame = CreateFrame("FRAME", "CaerdonSpecBrokerFrame")
eventFrame:RegisterEvent "ADDON_LOADED"
eventFrame:SetScript("OnEvent", OnEvent)

specObject = CreateDataObject()
